"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
function readPackageJson(folder = process.cwd()) {
    try {
        return JSON.parse(fs_1.readFileSync(path_1.default.join(folder, "package.json"), {
            encoding: "utf-8"
        }));
    }
    catch (e) {
        throw new Error('Input package.json file does not exist or has bad format, check "inputFolder" option');
    }
}
function writePackageJson(folder, contents) {
    try {
        fs_1.writeFileSync(path_1.default.join(folder, "package.json"), JSON.stringify(contents, null, 2), {
            encoding: "utf-8"
        });
    }
    catch (e) {
        throw new Error('Unable to save generated package.json file, check "outputFolder" option');
    }
}
function normalizeImportModules(imports) {
    return imports.map(module => {
        const pathParts = module.split(/[/\\]/);
        return pathParts[0][0] === "@"
            ? `${pathParts[0]}/${pathParts[1]}`
            : pathParts[0];
    });
}
function generatePackageJson(options = {}) {
    const { additionalDependencies = [], baseContents = {}, inputFolder = process.cwd(), outputFolder } = options;
    return {
        name: "generate-package-json",
        generateBundle: async (outputOptions, bundle) => {
            const inputFile = readPackageJson(inputFolder);
            const outputPath = outputFolder || outputOptions.dir || path_1.default.dirname(outputOptions.file);
            let dependencies = [];
            Object.values(bundle).forEach((chunk) => {
                if (chunk.imports) {
                    dependencies = [
                        ...dependencies,
                        ...normalizeImportModules(chunk.imports)
                    ];
                }
            });
            dependencies = Array.from(new Set([
                ...dependencies,
                ...(Array.isArray(additionalDependencies)
                    ? additionalDependencies
                    : Object.keys(additionalDependencies))
            ])).sort();
            const inputFileDependencies = inputFile.dependencies;
            const generatedDependencies = {};
            dependencies.forEach(dependency => {
                if (inputFileDependencies && inputFileDependencies[dependency]) {
                    generatedDependencies[dependency] = inputFileDependencies[dependency];
                }
                if (!Array.isArray(additionalDependencies) &&
                    additionalDependencies[dependency]) {
                    generatedDependencies[dependency] =
                        additionalDependencies[dependency];
                }
            });
            const generatedContents = {
                ...(typeof baseContents === "function"
                    ? baseContents(inputFile)
                    : baseContents),
                ...(Object.keys(generatedDependencies).length && {
                    dependencies: generatedDependencies
                })
            };
            writePackageJson(outputPath, generatedContents);
        }
    };
}
exports.default = generatePackageJson;
