"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const getBuiltins = require("builtins");
const readPkg = require("read-pkg");
const safeResolve = require("safe-resolve");
exports.default = ({ builtins = true, dependencies = true, peerDependencies = true } = {}) => ({
    name: "auto-external",
    options(opts) {
        const pkg = readPkg.sync({
            cwd: process.cwd(),
            normalize: false
        });
        let ids = [];
        if (dependencies && pkg.dependencies) {
            ids = ids.concat(Object.keys(pkg.dependencies));
        }
        if (peerDependencies && pkg.peerDependencies) {
            ids = ids.concat(Object.keys(pkg.peerDependencies));
        }
        if (builtins) {
            ids = ids.concat(getBuiltins());
        }
        ids = ids
            .map(idx => safeResolve(idx))
            .filter(val => Boolean(val));
        const external = (id) => {
            if (typeof opts.external === "function" && opts.external(id)) {
                return true;
            }
            if (Array.isArray(opts.external) && opts.external.includes(id)) {
                return true;
            }
            /**
             * The `id` argument is a resolved path if `rollup-plugin-node-resolve`
             * and `rollup-plugin-commonjs` are included.
             */
            const resolvedPath = safeResolve(id);
            if (resolvedPath === null) {
                return false;
            }
            const resolvedDirname = path_1.dirname(resolvedPath);
            return ids.some(idx => resolvedDirname.startsWith(path_1.dirname(idx)));
        };
        return Object.assign({}, opts, { external });
    }
});
